import c4d
import os


#v 1.0 
#v 1.01 ajout de l'undo.


from c4d import plugins, utils, bitmaps, gui, documents
# be sure to use a unique ID obtained from www.plugincafe.com
PLUGIN_ID = 1028518
dlg_pyresize		  		 =   10001
DIALOG_TITLE                 =   10002
ID_VSIZEX                    =   10003
ID_VSIZEY                    =   10004
ID_VSIZEZ                    =   10005
SIZEGROUP                    =   10006



class ResizeObjectDialog(gui.GeDialog):
    actualSize = c4d.Vector(0.0,0.0,0.0)
    
    def UpdateSize(self):
        self.doc = c4d.documents.GetActiveDocument()
        op = self.doc.GetActiveObject()
        if op:
            opsize = op.GetRad()*2
            opscale  = op.GetAbsScale()
            newsize = c4d.Vector(0.0,0.0,0.0)
            newsize.x = opsize.x*opscale.x
            newsize.y = opsize.y*opscale.y
            newsize.z = opsize.z*opscale.z
            self.SetReal(ID_VSIZEX, newsize.x)
            self.SetReal(ID_VSIZEY, newsize.y)
            self.SetReal(ID_VSIZEZ, newsize.z)
            self.actualSize = newsize
        
    def ChangeSize(self):
        # vérifier que l'objet sélectionné est du type polygon. 
        self.doc = c4d.documents.GetActiveDocument()
        op = self.doc.GetActiveObject()
        if not op: return False
        if not (op.CheckType(c4d.Opolygon)): return False
        Ratio = c4d.Vector(1.0,1.0,1.0)
        Ratio.x = self.GetReal(ID_VSIZEX) / self.actualSize.x
        Ratio.y = self.GetReal(ID_VSIZEY) / self.actualSize.y
        Ratio.z = self.GetReal(ID_VSIZEZ) / self.actualSize.z

        # assigner le ratio aux autres champs    
        opscale = op.GetAbsScale()
        if (round(Ratio.x,5)!= 1.0):
            Ratio = c4d.Vector(Ratio.x, Ratio.x, Ratio.x)
            
        elif (round(Ratio.y,5) <> 1.0):
            Ratio = c4d.Vector(Ratio.y, Ratio.y, Ratio.y)
        elif (round(Ratio.z,5) <> 1.0):
            Ratio = c4d.Vector(Ratio.z, Ratio.z, Ratio.z)

        # changer les coordonnées des points en fonction du ratio
        pcnt = op.GetPointCount()
        newPoint = c4d.Vector(0,0,0)
        self.doc.StartUndo()        
        for cpt in range (0, pcnt):
                oldPoint = op.GetPoint(cpt)
                newPoint = oldPoint * Ratio.x
                self.doc.AddUndo(c4d.UNDOTYPE_CHANGE, op)
                op.SetPoint(cpt, newPoint)
                            
        
        self.doc.EndUndo()
        op.Message(c4d.MSG_UPDATE)

        return True
    
    
    
    def CreateLayout(self):
        
        
        
        return self.LoadDialogResource(dlg_pyresize)
           
    def CoreMessage(self, id, msg):
        if id ==c4d.EVMSG_CHANGE:
            
            self.UpdateSize()
        
        return True
    
    def InitValues(self):
        self.doc = c4d.documents.GetActiveDocument()
        if not self.doc: return False
        op = self.doc.GetActiveObject()
        if self.doc:
            op = self.doc.GetActiveObject()
            if not op: return False
            opsize = op.GetRad()*2
            opscale  = op.GetAbsScale()
            if (op):
                newsize = c4d.Vector(0.0,0.0,0.0)
                newsize.x = opsize.x*opscale.x
                newsize.y = opsize.y*opscale.y
                newsize.z = opsize.z*opscale.z

                self.SetReal(ID_VSIZEX, newsize.x)
                self.SetReal(ID_VSIZEY, newsize.y)
                self.SetReal(ID_VSIZEZ, newsize.z)
                self.actualSize = newsize
        else:
            self.SetReal(ID_VSIZEX, 0.0)
            self.SetReal(ID_VSIZEY, 0.0)
            self.SetReal(ID_VSIZEZ, 0.0)
            self.actualSize = c4d.Vector(0.0,0.0,0.0)
        return True

    def Command(self, id, msg):
        if id == ID_VSIZEX:
            self.ChangeSize()
            c4d.EventAdd()
        elif id == ID_VSIZEY:
            self.ChangeSize()
            c4d.EventAdd()
        elif id == ID_VSIZEZ:
            self.ChangeSize()
            c4d.EventAdd()  
        return True

    
    

class ResizeObject (plugins.CommandData):
    dialog = None;
    
    def Execute (self, doc):
        if self.dialog is None:
            self.dialog = ResizeObjectDialog()
        return self.dialog.Open(dlgtype = c4d.DLG_TYPE_ASYNC, pluginid = PLUGIN_ID, defaultw = 400, defaulth = 32)
    

    
    
    def RestoreLayout(self, sec_ref):
        if self.dialog is None:
            self.dialog = ResizeObjectDialog()
        return self.dialog.Restore(pluginid = PLUGIN_ID, secret = sec_ref)







if __name__ == "__main__":
    # load icon.tif from res into bmp
    bmp = bitmaps.BaseBitmap()
    dir, file = os.path.split(__file__)
    fn = os.path.join(dir, "res", "icon.tif")
    bmp.InitWith(fn)
    # register the plugin
    plugins.RegisterCommandPlugin(id=PLUGIN_ID, 
                                  str="Py-resize",
                                  info=0,
                                  help="dlg_pyresize", 
                                  dat=ResizeObject(),
                                  icon=bmp)
